"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const iam_1 = require("./iam");
const file_1 = require("../utils/file");
const varset_1 = require("../utils/varset");
async function getPolicyDefaultVersion(policy) {
    const params = {
        PolicyArn: policy.Arn,
        VersionId: policy.DefaultVersionId
    };
    const data = await iam_1.iam.getPolicyVersion(params).promise();
    const result = data.PolicyVersion;
    return {
        name: policy.PolicyName,
        arn: params.PolicyArn,
        document: result.Document,
        versionId: result.VersionId,
        isDefaultVersion: result.IsDefaultVersion,
        createDate: result.CreateDate,
    };
}
exports.getPolicyDefaultVersion = getPolicyDefaultVersion;
async function getPolicyDefaultWithVersionInfoByArn(arn) {
    const versions = await listPolicyVersions(arn);
    if (versions.length === 0) {
        throw new Error('Failed to get policy versions');
    }
    let defaultId = '';
    let oldestId = '';
    let createDate = new Date('2099-12-31T00:00:00.000Z');
    versions.forEach((ver) => {
        if (ver.IsDefaultVersion) {
            defaultId = ver.VersionId;
        }
        else {
            const date = ver.CreateDate;
            if (date < createDate) {
                createDate = date;
                oldestId = ver.VersionId;
            }
        }
    });
    return {
        defaultId,
        oldestId,
        count: versions.length,
        currentPolicy: await getPolicyDefaultVersion({ Arn: arn, DefaultVersionId: defaultId })
    };
}
exports.getPolicyDefaultWithVersionInfoByArn = getPolicyDefaultWithVersionInfoByArn;
async function listPolicyVersions(arn) {
    const params = {
        PolicyArn: arn,
        MaxItems: 10,
    };
    const data = await iam_1.iam.listPolicyVersions(params).promise();
    return data.Versions || [];
}
exports.listPolicyVersions = listPolicyVersions;
async function createPolicy({ name, document }, indent) {
    const params = {
        PolicyName: name,
        PolicyDocument: convertDocToJSON(document, indent),
    };
    return iam_1.iam.createPolicy(params).promise();
}
exports.createPolicy = createPolicy;
async function createPolicyDefaultVersion(arn, doc, indent) {
    const params = {
        PolicyArn: arn,
        PolicyDocument: doc,
        SetAsDefault: true,
    };
    return iam_1.iam.createPolicyVersion(params).promise();
}
exports.createPolicyDefaultVersion = createPolicyDefaultVersion;
async function deletePolicyVersion(arn, versionId) {
    const params = {
        PolicyArn: arn,
        VersionId: versionId,
    };
    return iam_1.iam.deletePolicyVersion(params).promise();
}
exports.deletePolicyVersion = deletePolicyVersion;
let _policyArnPrefix = null;
async function getPolicyArnPrefix() {
    if (_policyArnPrefix) {
        return _policyArnPrefix;
    }
    const params = {
        Scope: 'Local',
        MaxItems: 1,
    };
    const data = await iam_1.iam.listPolicies(params).promise();
    const policies = data.Policies;
    if (policies.length) {
        _policyArnPrefix = policies[0].Arn.split('/')[0];
    }
    return _policyArnPrefix;
}
exports.getPolicyArnPrefix = getPolicyArnPrefix;
async function readPolicyFile(filePath, varSet) {
    let name = '';
    try {
        name = path_1.default.basename(filePath, '.json');
        const text = await file_1.readFile(filePath);
        return {
            name: varset_1.substitute(name, varSet),
            document: varset_1.parseJSON(text, varSet),
        };
    }
    catch (err) {
        console.error(`Failed to read ${name}`);
        throw err;
    }
}
exports.readPolicyFile = readPolicyFile;
function convertDocToJSON(doc, indent = 4) {
    return JSON.stringify(doc, null, indent);
}
exports.convertDocToJSON = convertDocToJSON;
