"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsonDiff = require('json-diff');
const role_1 = require("../aws/role");
const attach_1 = require("../aws/attach");
const role_2 = require("../aws/role");
const result_1 = require("../utils/result");
class InvalidRoleError extends Error {
    constructor(msg, target, diff) {
        super(msg);
        this.target = target;
        this.diff = diff || null;
    }
}
exports.InvalidRoleError = InvalidRoleError;
class RoleValidator {
    constructor(opts = {}) {
        this._color = !(opts['plain'] || false);
        this._invalidCnt = 0;
    }
    async validate({ name, document }) {
        try {
            const role = document.Role;
            const roleName = role.RoleName;
            await this._validateRoleOrRoleWithInstanceProfile(role);
            let results = await this._validateAttachedPolicies(roleName, document.AttachedPolicies);
            if (results)
                return results;
            return [result_1.OK(roleName)];
        }
        catch (err) {
            if (err instanceof InvalidRoleError) {
                this._invalidCnt++;
                return [result_1.NG(err.message, err.target, err.diff)];
            }
            return [result_1.NG('Role: %1 is invalid JSON format.', name)];
        }
    }
    async _validateRoleOrRoleWithInstanceProfile(definedRole) {
        const roleName = definedRole.RoleName;
        let currentRole;
        if (role_2.isEc2Role(definedRole)) {
            const profile = await role_2.getInstanceProfile(roleName);
            if (!profile) {
                throw new InvalidRoleError('%1 instance profile does not exist.', roleName);
            }
            currentRole = profile.Roles[0];
            currentRole.AssumeRolePolicyDocument = JSON.parse(decodeURIComponent(currentRole.AssumeRolePolicyDocument));
        }
        else {
            const gotRole = await role_2.getRole(roleName);
            if (!gotRole) {
                throw new InvalidRoleError('%1 does not exist.', roleName);
            }
            currentRole = gotRole;
        }
        const current = this._convertForDiff(currentRole);
        const defined = this._convertForDiff(definedRole);
        const df = jsonDiff.diffString(current, defined, { color: this._color });
        if (df) {
            throw new InvalidRoleError('%1 is invalid.', roleName, df);
        }
    }
    async _validateAttachedPolicies(roleName, attachedPolicies) {
        const currentPolicies = await role_1.getAttachedPoliciesByRole(roleName);
        const results = [];
        const definedPolicies = attachedPolicies;
        definedPolicies.forEach((definedPolicy) => {
            if (!attach_1.containPolicy(currentPolicies, definedPolicy)) {
                results.push(result_1.NG('Role: %1 does not have Policy: %2.', [roleName, definedPolicy.PolicyName]));
            }
        });
        currentPolicies.forEach((currentPolicy) => {
            if (!attach_1.containPolicy(definedPolicies, currentPolicy)) {
                results.push(result_1.NG('Role: %1 have Policy: %2 not required.', [roleName, currentPolicy.PolicyName]));
            }
        });
        if (results.length) {
            this._invalidCnt++;
            return results;
        }
        return null;
    }
    isValid() {
        return this._invalidCnt === 0;
    }
    _convertForDiff(role) {
        const data = JSON.parse(JSON.stringify(role));
        delete data.Arn;
        delete data.RoleId;
        delete data.CreateDate;
        delete data.MaxSessionDuration;
        delete data.Tags;
        return data;
    }
}
exports.RoleValidator = RoleValidator;
